@@ -1,8 +1,12 @@
 from typing import List, Optional, Union
 
+from easydata.exceptions import DropItem
+
 # Config attributes used by multiple parsers
 ED_LANGUAGE: str = "en"
 
+ED_DROP_ITEM_EXCEPTION = DropItem
+
 # Config attributes used by date time parsers
 ED_DATETIME_FORMAT: str = "%m/%d/%Y %H:%M:%S"
 
@@ -23,6 +27,13 @@ ED_URL_DOMAIN: Optional[str] = None
 
 ED_URL_PROTOCOL: str = "https"
 
+# Config attributes used by number parsers
+ED_NUMBER_DECIMALS: Union[int, bool] = False
+
+ED_NUMBER_MIN_VALUE: Optional[Union[int, float]] = None
+
+ED_NUMBER_MAX_VALUE: Optional[Union[int, float]] = None
+
 # Config attributes used by price parsers
 ED_PRICE_DECIMALS: Union[int, bool] = 2
 
@@ -45,10 +56,3 @@ ED_ITEM_DISCOUNT_DECIMALS: int = 2
 ED_ITEM_DISCOUNT_NO_DECIMALS: bool = False
 
 ED_ITEM_DISCOUNT_REMOVE_ITEM_SALE_PRICE_KEY: bool = False
-
-# Config attributes used by models
-ED_DATA_VARIANTS_NAME = "variants"
-
-ED_DATA_VARIANTS_KEY_NAME = "variants_key"
-
-ED_DATA_VARIANT_NAME = "variant"
