@@ -1,13 +1,16 @@
 import json
 from abc import ABC, abstractmethod
-from typing import Any, Dict, List, Optional, Union
+from functools import lru_cache
+from typing import Any, Dict, Iterator, List, Optional, Union
 
 import xmltodict
 import yaml
+from easytxt.text import replace_chars_by_keys
 from pyquery import PyQuery
 
 from easydata.data import DataBag
 from easydata.parsers.base import BaseData
+from easydata.parsers.data import Data
 from easydata.processors.base import BaseProcessor
 from easydata.queries.base import QuerySearch
 from easydata.queries.re import ReQuery
@@ -19,17 +22,22 @@ __all__ = (
     "DataToPqProcessor",
     "DataJsonToDictProcessor",
     "DataJsonFromQueryToDictProcessor",
+    "DataFromIterQueryProcessor",
+    "DataYamlToDictProcessor",
+    "DataXmlToDictProcessor",
     "DataTextFromReProcessor",
     "DataJsonFromReToDictProcessor",
     "DataFromQueryProcessor",
-    "DataVariantProcessor",
+    "DataVariantsProcessor",
 )
 
 
 class DataBaseProcessor(BaseProcessor, ABC):
+    _multi: bool = False
+
     def __init__(
         self,
-        source: str = "data",
+        source: str = "main",
         new_source: Optional[str] = None,
         process_source_data=None,
     ):
@@ -38,15 +46,36 @@ class DataBaseProcessor(BaseProcessor, ABC):
         self._new_source = new_source
         self._process_source_data = process_source_data
 
-    def parse(self, data: DataBag) -> DataBag:
-        new_source = self._new_source
-
+    def parse(self, data: DataBag) -> Iterator[DataBag]:
         source_data = data[self._source]
 
         if self._process_source_data:
             source_data = self._process_source_data(source_data)
 
-        trans_data = self._process_data(source_data)
+        transformed_data = self._process_data(source_data)
+
+        if self._multi:
+            for iter_transformed_data in transformed_data:
+                data_copy = data.copy()
+
+                yield self._transformed_data_to_data(iter_transformed_data, data_copy)
+        else:
+            yield self._transformed_data_to_data(transformed_data, data)
+
+    def parse_data(self, data=None, **kwargs):
+        if data:
+            kwargs["main"] = data
+
+        data = DataBag(**kwargs)
+
+        return self.parse(data)
+
+    @abstractmethod
+    def _process_data(self, source_data) -> Any:
+        pass
+
+    def _transformed_data_to_data(self, transformed_data, data):
+        new_source = self._new_source
 
         if new_source is None:
             original_source = "{}_raw".format(self._source)
@@ -54,22 +83,10 @@ class DataBaseProcessor(BaseProcessor, ABC):
 
             new_source = self._source
 
-        data[new_source] = trans_data
+        data[new_source] = transformed_data
 
         return data
 
-    def parse_data(self, data=None, **kwargs):
-        if data:
-            kwargs["data"] = data
-
-        data = DataBag(self, **kwargs)
-
-        return self.parse(data)
-
-    @abstractmethod
-    def _process_data(self, source_data) -> Any:
-        pass
-
 
 class DataProcessor(DataBaseProcessor):
     def _process_data(self, source_data) -> Any:
@@ -169,12 +186,11 @@ class DataXmlToDictProcessor(DataBaseProcessor):
 class DataFromQueryProcessor(DataBaseProcessor):
     def __init__(
         self,
-        query: Union[List[QuerySearch], QuerySearch],
+        query: Union[QuerySearch],
         **kwargs,
     ):
 
-        if isinstance(query, QuerySearch):
-            self._query = [query]
+        self._query = query
 
         super().__init__(**kwargs)
 
@@ -182,6 +198,10 @@ class DataFromQueryProcessor(DataBaseProcessor):
         return parse.query_search(self._query, data)
 
 
+class DataFromIterQueryProcessor(DataFromQueryProcessor):
+    _multi = True
+
+
 class DataJsonFromQueryToDictProcessor(DataFromQueryProcessor):
     def process_data(self, data: Any) -> Any:
         jt = super()._process_data(data)
@@ -197,6 +217,7 @@ class DataTextFromReProcessor(DataBaseProcessor):
         dotall=True,
         ignore_case=False,
         bytes_to_string_decode: str = "utf-8",
+        replace_keys: Optional[list] = None,
         none_if_empty=False,
         process_value=None,
         **kwargs,
@@ -206,6 +227,7 @@ class DataTextFromReProcessor(DataBaseProcessor):
         self._dotall = dotall
         self._ignore_case = ignore_case
         self._bytes_to_string_decode = bytes_to_string_decode
+        self._replace_keys = replace_keys
         self._none_if_empty = none_if_empty
         self._process_value = process_value
 
@@ -232,6 +254,9 @@ class DataTextFromReProcessor(DataBaseProcessor):
             else:
                 value = self._process_value(value)
 
+        if value and self._replace_keys:
+            value = replace_chars_by_keys(value, self._replace_keys)
+
         return value
 
 
@@ -248,75 +273,65 @@ class DataJsonFromReToDictProcessor(DataTextFromReProcessor):
         return json.loads(value)
 
 
-class DataVariantProcessor(DataBaseProcessor):
+class DataVariantsProcessor(DataBaseProcessor):
     def __init__(
         self,
-        source: str = "data",
-        parser: Optional[BaseData] = None,
-        variant_parser: Optional[BaseData] = None,
         query: Optional[QuerySearch] = None,
-        variant_query: Optional[QuerySearch] = None,
-        multi_values: bool = False,
-        with_variant_values: bool = True,
-        variant_values_lower: bool = True,
+        source: str = "main",
+        parser: Optional[BaseData] = None,
+        key_parser: Optional[BaseData] = None,
+        key_query: Optional[QuerySearch] = None,
         **kwargs,
     ):
 
-        if variant_parser and variant_query:
+        if key_parser and key_query:
             error_msg = (
-                "variant_query or variant_parser attributes cannot "
-                "be set at the same time."
+                "key_parser or key_query attributes cannot " "be set at the same time."
             )
             raise AttributeError(error_msg)
 
-        kwargs["source"] = source
-
-        if "new_source" not in kwargs:
-            kwargs["new_source"] = "{}_variants".format(kwargs["source"])
-
-        self._parser = parser
-        self._variant_parser = variant_parser
         self._query = query
-        self._variant_query = variant_query
-        self._multi_values = multi_values
-        self._with_variant_values = with_variant_values
-        self._variant_values_lower = variant_values_lower
+        self._key_parser = key_parser
+        self._key_query = key_query
 
-        super().__init__(**kwargs)
+        self.__parser = parser
 
-    def _process_data(self, data: Any) -> Any:
-        if self._parser:
-            data = self._parser.parse(data)
+        super().__init__(
+            source=source,
+            **kwargs,
+        )
 
-        if self._query:
-            data = self._query.get(data)
+    def parse(self, data: DataBag) -> Iterator[DataBag]:
+        variants_source = self._new_source or self._source
 
+        for iter_data in super(DataVariantsProcessor, self).parse(data):
+            for var_iter_data in parse.variants_data(iter_data, variants_source):
+                yield var_iter_data
+
+    @property  # type: ignore
+    @lru_cache(maxsize=None)
+    def _parser(self) -> Optional[BaseData]:
+        return Data(self._query) if self._query else self.__parser
+
+    def _process_data(self, data: Any) -> Dict[Optional[str], Any]:
         variants_data: Dict[Optional[str], Any] = {}
 
-        for data_info in data:
-            variant_key = self._get_variant_key(data_info)
+        parser = self._parser
 
-            if self._with_variant_values and self._variant_values_lower:
-                if variant_key:
-                    variant_key = variant_key.lower()
+        if parser:
+            data = parser.init_config(self.config).parse(data)  # type: ignore
 
-            if variant_key not in variants_data:
-                variants_data[variant_key] = []
+        for data_index, data_info in enumerate(data):
+            if self._key_parser:
+                variant_group_key = self._key_parser.parse(data_info)
+            elif self._key_query:
+                variant_group_key = self._key_query.get(data_info)
+            else:
+                variant_group_key = str(data_index)
 
-            variants_data[variant_key].append(data_info)
+            if variant_group_key not in variants_data:
+                variants_data[variant_group_key] = []
 
-        if not self._multi_values:
-            variants_data = {k: v[0] for k, v in variants_data.items() if v}
+            variants_data[variant_group_key].append(data_info)
 
-        if self._with_variant_values:
-            return variants_data
-
-        return list(variants_data.values())
-
-    def _get_variant_key(self, data: Any) -> Optional[str]:
-        if self._variant_parser:
-            return self._variant_parser.parse(data)
-        elif self._variant_query:
-            return self._variant_query.get(data)
-
-        return None
+        return variants_data
