@@ -5,10 +5,18 @@ from easydata.utils import validate
 
 
 class QuerySearch(ABC):
+    def __init__(
+        self,
+        query: str = None,
+    ):
+
+        self._query = query
+
     def get(
         self,
         data: Any,
-        source: str = "data",
+        source: str = "main",
+        query_params: Optional[dict] = None,
     ) -> Any:
 
         validate.if_data_bag_with_source(
@@ -21,12 +29,21 @@ class QuerySearch(ABC):
 
         data = self._process_data(data, source)
 
-        return self._parse(data)
+        if self._query and query_params:
+            query = self._apply_query_params(self._query, query_params)
+        else:
+            query = self._query
+
+        return self._parse(data, query)
+
+    def _apply_query_params(self, query, query_params):
+        return query.format(**query_params)
 
     @abstractmethod
     def _parse(
         self,
         data: Any,
+        query: Optional[str],
     ):
         pass
 
